//+------------------------------------------------------------------+
//|                                               Harami Boy 1  .mq5 |
//|                                       Copyright 2021, Brian Jaka |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, Brian Jaka"
#property link      "https://www.mql5.com"
#property version   "1.10"
/*
 This EA trades a Harami setup at open of Bar[0]
 BUY - One bearish and one bullish bar formation
 SELL- One bullish and one bearish bar formation
*/
#include<Trade\SymbolInfo.mqh>
#include<Trade\Trade.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>
//---
CSymbolInfo    m_symbol;                     // object of CSymbolInfo class for receiving symbol settings
CTrade         m_trade;                      // object of CTrade class for performing trade operations
CPositionInfo  m_position;                   // object of CPositionInfo class for receiving position settings
CAccountInfo   m_account;                    // object of CAccountInfo class for receiving account settings
//--- input parameters
input group             "Trading settings"
input uint                 InpStopLoss             = 150;            // Stop Loss
input uint                 InpTakeProfit           = 460;            // Take Profit
input group             "Position size management (lot calculation)"
input double               InpLots                 = 0.01;           // Lots\
input group             "Additional features"
input bool                 InpPrintLog             = true;           // Print log
input ulong                InpDeviation            = 10;             // Deviation, in Points (1.00045-1.00055=10 points)
input ulong                InpMagic                = 12345;          // Magic number
//---
double   m_stop_loss                = 0.0;      // Stop Loss                  -> double
double   m_take_profit              = 0.0;      // Take Profit                -> double
datetime m_prev_bars                = 0;        // "0" -> D'1970.01.01 00:00';
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- set the name for the appropriate symbol
   ResetLastError();
   if(!m_symbol.Name(Symbol())) // sets symbol name
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: CSymbolInfo.Name");
      return(INIT_FAILED);
     }
   RefreshRates();
//---
   m_trade.SetExpertMagicNumber(InpMagic);            //--- set magicNumber for your orders identification
   m_trade.SetMarginMode();                           //--- set margin mode for your orders
   m_trade.SetTypeFillingBySymbol(m_symbol.Name());   //--- set order filling mode allowed by the server
   m_trade.SetDeviationInPoints(InpDeviation);        //--- set available slippage in points when buying/selling
//---
   m_stop_loss                = InpStopLoss                 * m_symbol.Point();
   m_take_profit              = InpTakeProfit               * m_symbol.Point();
//--- receive current rates and display
   Print(m_symbol.Name()," (",m_symbol.Description(),")","  Bid=",m_symbol.Bid(),"   Ask=",m_symbol.Ask());
//--- receive minimum freeze levels for trade operations
   Print("StopsLevel=",m_symbol.StopsLevel()," points, FreezeLevel=",m_symbol.FreezeLevel()," points");
//--- receive the number of decimal places and point size
   Print("Digits=",m_symbol.Digits(),", Point=",DoubleToString(m_symbol.Point(),m_symbol.Digits()));
//--- spread info
   Print("SpreadFloat=",m_symbol.SpreadFloat(),", Spread(current)=",m_symbol.Spread()," points");
//--- request order execution type for limitations
   Print("Limitations for trade operations: ",EnumToString(m_symbol.TradeMode())," (",m_symbol.TradeModeDescription(),")");
//--- clarifying trades execution mode
   Print("Trades execution mode: ",EnumToString(m_symbol.TradeExecution())," (",m_symbol.TradeExecutionDescription(),")");
//--- clarifying contracts price calculation method
   Print("Contract price calculation: ",EnumToString(m_symbol.TradeCalcMode())," (",m_symbol.TradeCalcModeDescription(),")");
//--- sizes of contracts
   Print("Standard contract size: ",m_symbol.ContractSize()," (",m_symbol.CurrencyBase(),")");
//--- minimum and maximum volumes in trade operations
   Print("Volume info: LotsMin=",m_symbol.LotsMin(),"  LotsMax=",m_symbol.LotsMax(),"  LotsStep=",m_symbol.LotsStep());
//---
   Print(__FUNCTION__,"  completed");
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- work only at the time of the birth of new bar
   datetime time_0=iTime(m_symbol.Name(),Period(),0);
   if(time_0==m_prev_bars)
      return;
   m_prev_bars=time_0;
   if(!RefreshRates())
     {
      m_prev_bars=0;
      return;
     }
//---
   MqlRates rates[];
   ArraySetAsSeries(rates,true);
   int start_pos=0,count=6;
   if(CopyRates(m_symbol.Name(),Period(),start_pos,count,rates)!=count)
     {
      m_prev_bars=0;
      return;
     }
//--- Condition for opening BUY position
   if(rates[1].open-rates[1].close<0.0)
      if(rates[2].open-rates[2].close>0.0)
        {
         //--- check volume before OrderSend to avoid "not enough money" error (CTrade)
         double free_margin_check=m_account.FreeMarginCheck(m_symbol.Name(),
                                  ORDER_TYPE_BUY,
                                  InpLots,
                                  m_symbol.Bid());
         double margin_check=m_account.MarginCheck(m_symbol.Name(),
                             ORDER_TYPE_BUY,
                             InpLots,
                             m_symbol.Bid());
         if(free_margin_check>margin_check)
           {
            double sl=(m_stop_loss==0.0)?0.0:m_symbol.Bid()+m_stop_loss;
            double tp=(m_take_profit==0.0)?0.0:m_symbol.Bid()-m_take_profit;
            //---
            if(InpPrintLog)
               Print(__FILE__," ",__FUNCTION__,", OK: ","Signal SELL");
            m_trade.Sell(InpLots,m_symbol.Name(),m_symbol.Bid(),sl,tp);
           }
        }
//---
  }
//+------------------------------------------------------------------+
//| Refreshes the symbol quotes data                                 |
//+------------------------------------------------------------------+
bool RefreshRates()
  {
//--- refresh rates
   if(!m_symbol.RefreshRates())
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: ","RefreshRates error");
      return(false);
     }
//--- protection against the return value of "zero"
   if(m_symbol.Ask()==0 || m_symbol.Bid()==0)
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: ","Ask == 0.0 OR Bid == 0.0");
      return(false);
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
